@@ -110,17 +110,17 @@ public class DefaultNHttpClientConnection
      * @param buffersize buffer size. Must be a positive number.
      * @param fragmentSizeHint fragment size hint.
      * @param allocator memory allocator.
-     *   If <code>null</code> {@link org.apache.http.nio.util.HeapByteBufferAllocator#INSTANCE}
+     *   If {@code null} {@link org.apache.http.nio.util.HeapByteBufferAllocator#INSTANCE}
      *   will be used.
      * @param chardecoder decoder to be used for decoding HTTP protocol elements.
-     *   If <code>null</code> simple type cast will be used for byte to char conversion.
+     *   If {@code null} simple type cast will be used for byte to char conversion.
      * @param charencoder encoder to be used for encoding HTTP protocol elements.
-     *   If <code>null</code> simple type cast will be used for char to byte conversion.
-     * @param constraints Message constraints. If <code>null</code>
+     *   If {@code null} simple type cast will be used for char to byte conversion.
+     * @param constraints Message constraints. If {@code null}
      *   {@link MessageConstraints#DEFAULT} will be used.
-     * @param incomingContentStrategy incoming content length strategy. If <code>null</code>
+     * @param incomingContentStrategy incoming content length strategy. If {@code null}
      *   {@link org.apache.http.impl.entity.LaxContentLengthStrategy#INSTANCE} will be used.
-     * @param outgoingContentStrategy outgoing content length strategy. If <code>null</code>
+     * @param outgoingContentStrategy outgoing content length strategy. If {@code null}
      *   {@link org.apache.http.impl.entity.StrictContentLengthStrategy#INSTANCE} will be used.
      *
      * @since 4.3
@@ -137,8 +137,8 @@ public class DefaultNHttpClientConnection
             final ContentLengthStrategy outgoingContentStrategy,
             final NHttpMessageWriterFactory<HttpRequest> requestWriterFactory,
             final NHttpMessageParserFactory<HttpResponse> responseParserFactory) {
-        super(session, buffersize, fragmentSizeHint,  allocator, chardecoder, charencoder,
-                incomingContentStrategy, outgoingContentStrategy);
+        super(session, buffersize, fragmentSizeHint, allocator, chardecoder, charencoder,
+                constraints, incomingContentStrategy, outgoingContentStrategy);
         this.requestWriter = (requestWriterFactory != null ? requestWriterFactory :
             DefaultHttpRequestWriterFactory.INSTANCE).create(this.outbuf);
         this.responseParser = (responseParserFactory != null ? responseParserFactory :
@@ -217,12 +217,14 @@ public class DefaultNHttpClientConnection
     protected void onRequestSubmitted(final HttpRequest request) {
     }
 
+    @Override
     public void resetInput() {
         this.response = null;
         this.contentDecoder = null;
         this.responseParser.reset();
     }
 
+    @Override
     public void resetOutput() {
         this.request = null;
         this.contentEncoder = null;
@@ -250,13 +252,14 @@ public class DefaultNHttpClientConnection
                         this.response.setEntity(entity);
                         this.connMetrics.incrementResponseCount();
                     }
+                    this.hasBufferedInput = this.inbuf.hasData();
                     onResponseReceived(this.response);
                     handler.responseReceived(this);
                     if (this.contentDecoder == null) {
                         resetInput();
                     }
                 }
-                if (bytesRead == -1) {
+                if (bytesRead == -1 && !this.inbuf.hasData()) {
                     handler.endOfInput(this);
                 }
             }
@@ -316,6 +319,7 @@ public class DefaultNHttpClientConnection
         }
     }
 
+    @Override
     public void submitRequest(final HttpRequest request) throws IOException, HttpException {
         Args.notNull(request, "HTTP request");
         assertNotClosed();
@@ -335,14 +339,17 @@ public class DefaultNHttpClientConnection
         this.session.setEvent(EventMask.WRITE);
     }
 
+    @Override
     public boolean isRequestSubmitted() {
         return this.request != null;
     }
 
+    @Override
     public void consumeInput(final NHttpClientHandler handler) {
         consumeInput(new NHttpClientEventHandlerAdaptor(handler));
     }
 
+    @Override
     public void produceOutput(final NHttpClientHandler handler) {
         produceOutput(new NHttpClientEventHandlerAdaptor(handler));
     }
