@@ -31,12 +31,16 @@ import java.io.IOException;
 import java.net.SocketTimeoutException;
 
 import org.apache.http.ConnectionClosedException;
+import org.apache.http.ExceptionLogger;
+import org.apache.http.HttpEntity;
 import org.apache.http.HttpEntityEnclosingRequest;
 import org.apache.http.HttpException;
 import org.apache.http.HttpRequest;
 import org.apache.http.HttpResponse;
 import org.apache.http.HttpStatus;
+import org.apache.http.HttpVersion;
 import org.apache.http.ProtocolException;
+import org.apache.http.ProtocolVersion;
 import org.apache.http.annotation.Immutable;
 import org.apache.http.nio.ContentDecoder;
 import org.apache.http.nio.ContentEncoder;
@@ -48,24 +52,24 @@ import org.apache.http.util.Args;
 import org.apache.http.util.Asserts;
 
 /**
- * <tt>HttpAsyncRequestExecutor</tt> is a fully asynchronous HTTP client side
+ * {@code HttpAsyncRequestExecutor} is a fully asynchronous HTTP client side
  * protocol handler based on the NIO (non-blocking) I/O model.
- * <tt>HttpAsyncRequestExecutor</tt> translates individual events fired through
+ * {@code HttpAsyncRequestExecutor} translates individual events fired through
  * the {@link NHttpClientEventHandler} interface into logically related HTTP
  * message exchanges.
- * <p/> The caller is expected to pass an instance of
+ * <p> The caller is expected to pass an instance of
  * {@link HttpAsyncClientExchangeHandler} to be used for the next series
  * of HTTP message exchanges through the connection context using
  * {@link #HTTP_HANDLER} attribute. HTTP exchange sequence is considered
  * complete when the {@link HttpAsyncClientExchangeHandler#isDone()} method
- * returns <code>true</code>. The {@link HttpAsyncRequester} utility class can
+ * returns {@code true}. The {@link HttpAsyncRequester} utility class can
  * be used to facilitate initiation of asynchronous HTTP request execution.
- * <p/>
- * Individual <tt>HttpAsyncClientExchangeHandler</tt> are expected to make use of
+ * <p>
+ * Individual {@code HttpAsyncClientExchangeHandler} are expected to make use of
  * a {@link org.apache.http.protocol.HttpProcessor} to generate mandatory protocol
  * headers for all outgoing messages and apply common, cross-cutting message
  * transformations to all incoming and outgoing messages.
- * <tt>HttpAsyncClientExchangeHandler</tt>s can delegate implementation of
+ * {@code HttpAsyncClientExchangeHandler}s can delegate implementation of
  * application specific content generation and processing to
  * a {@link HttpAsyncRequestProducer} and a {@link HttpAsyncResponseConsumer}.
  *
@@ -80,6 +84,25 @@ public class HttpAsyncRequestExecutor implements NHttpClientEventHandler {
     public static final String HTTP_HANDLER = "http.nio.exchange-handler";
 
     private final int waitForContinue;
+    private final ExceptionLogger exceptionLogger;
+
+    /**
+     * Creates new instance of {@code HttpAsyncRequestExecutor}.
+     * @param waitForContinue wait for continue time period.
+     * @param exceptionLogger Exception logger. If {@code null}
+     *   {@link ExceptionLogger#NO_OP} will be used. Please note that the exception
+     *   logger will be only used to log I/O exception thrown while closing
+     *   {@link java.io.Closeable} objects (such as {@link org.apache.http.HttpConnection}).
+     *
+     * @since 4.4
+     */
+    public HttpAsyncRequestExecutor(
+            final int waitForContinue,
+            final ExceptionLogger exceptionLogger) {
+        super();
+        this.waitForContinue = Args.positive(waitForContinue, "Wait for continue time");
+        this.exceptionLogger = exceptionLogger != null ? exceptionLogger : ExceptionLogger.NO_OP;
+    }
 
     /**
      * Creates new instance of HttpAsyncRequestExecutor.
@@ -87,14 +110,14 @@ public class HttpAsyncRequestExecutor implements NHttpClientEventHandler {
      * @since 4.3
      */
     public HttpAsyncRequestExecutor(final int waitForContinue) {
-        super();
-        this.waitForContinue = Args.positive(waitForContinue, "Wait for continue time");
+        this(waitForContinue, null);
     }
 
     public HttpAsyncRequestExecutor() {
-        this(DEFAULT_WAIT_FOR_CONTINUE);
+        this(DEFAULT_WAIT_FOR_CONTINUE, null);
     }
 
+    @Override
     public void connected(
             final NHttpClientConnection conn,
             final Object attachment) throws IOException, HttpException {
@@ -104,17 +127,23 @@ public class HttpAsyncRequestExecutor implements NHttpClientEventHandler {
         requestReady(conn);
     }
 
+    @Override
     public void closed(final NHttpClientConnection conn) {
         final State state = getState(conn);
         final HttpAsyncClientExchangeHandler handler = getHandler(conn);
+        if (state != null) {
+            if (state.getRequestState() != MessageState.READY || state.getResponseState() != MessageState.READY) {
+                if (handler != null) {
+                    handler.failed(new ConnectionClosedException("Connection closed unexpectedly"));
+                }
+            }
+        }
         if (state == null || (handler != null && handler.isDone())) {
             closeHandler(handler);
         }
-        if (state != null) {
-            state.reset();
-        }
     }
 
+    @Override
     public void exception(
             final NHttpClientConnection conn, final Exception cause) {
         shutdownConnection(conn);
@@ -126,70 +155,99 @@ public class HttpAsyncRequestExecutor implements NHttpClientEventHandler {
         }
     }
 
+    @Override
     public void requestReady(
             final NHttpClientConnection conn) throws IOException, HttpException {
-        final State state = ensureNotNull(getState(conn));
+        final State state = getState(conn);
+        Asserts.notNull(state, "Connection state");
+        Asserts.check(state.getRequestState() == MessageState.READY ||
+                        state.getRequestState() == MessageState.COMPLETED,
+                "Unexpected request state %s", state.getRequestState());
+
         if (state.getRequestState() != MessageState.READY) {
             return;
         }
-        HttpAsyncClientExchangeHandler handler = getHandler(conn);
-        if (handler != null && handler.isDone()) {
-            closeHandler(handler);
-            state.reset();
-            handler = null;
-        }
-        if (handler == null) {
+        final HttpAsyncClientExchangeHandler handler = getHandler(conn);
+        if (handler == null || handler.isDone()) {
             return;
         }
+        final boolean pipelined = handler.getClass().getAnnotation(Pipelined.class) != null;
 
         final HttpRequest request = handler.generateRequest();
         if (request == null) {
             return;
         }
+        final ProtocolVersion version = request.getRequestLine().getProtocolVersion();
+        if (pipelined && version.lessEquals(HttpVersion.HTTP_1_0)) {
+            throw new ProtocolException(version + " cannot be used with request pipelining");
+        }
         state.setRequest(request);
 
-        conn.submitRequest(request);
-
         if (request instanceof HttpEntityEnclosingRequest) {
-            if (((HttpEntityEnclosingRequest) request).expectContinue()) {
+            final boolean expectContinue = ((HttpEntityEnclosingRequest) request).expectContinue();
+            if (expectContinue && pipelined) {
+                throw new ProtocolException("Expect-continue handshake cannot be used with request pipelining");
+            }
+            conn.submitRequest(request);
+            if (expectContinue) {
                 final int timeout = conn.getSocketTimeout();
                 state.setTimeout(timeout);
                 conn.setSocketTimeout(this.waitForContinue);
                 state.setRequestState(MessageState.ACK_EXPECTED);
             } else {
-                state.setRequestState(MessageState.BODY_STREAM);
+                final HttpEntity entity = ((HttpEntityEnclosingRequest) request).getEntity();
+                if (entity != null) {
+                    state.setRequestState(MessageState.BODY_STREAM);
+                } else {
+                    handler.requestCompleted();
+                    state.setRequestState(pipelined ? MessageState.READY : MessageState.COMPLETED);
+                }
             }
         } else {
+            conn.submitRequest(request);
             handler.requestCompleted();
-            state.setRequestState(MessageState.COMPLETED);
+            state.setRequestState(pipelined ? MessageState.READY : MessageState.COMPLETED);
         }
     }
 
+    @Override
     public void outputReady(
             final NHttpClientConnection conn,
             final ContentEncoder encoder) throws IOException, HttpException {
-        final State state = ensureNotNull(getState(conn));
-        final HttpAsyncClientExchangeHandler handler = ensureNotNull(getHandler(conn));
+        final State state = getState(conn);
+        Asserts.notNull(state, "Connection state");
+        Asserts.check(state.getRequestState() == MessageState.BODY_STREAM ||
+                        state.getRequestState() == MessageState.ACK_EXPECTED,
+                "Unexpected request state %s", state.getRequestState());
+
+        final HttpAsyncClientExchangeHandler handler = getHandler(conn);
+        Asserts.notNull(handler, "Client exchange handler");
         if (state.getRequestState() == MessageState.ACK_EXPECTED) {
             conn.suspendOutput();
             return;
         }
         handler.produceContent(encoder, conn);
-        state.setRequestState(MessageState.BODY_STREAM);
         if (encoder.isCompleted()) {
             handler.requestCompleted();
-            state.setRequestState(MessageState.COMPLETED);
+            final boolean pipelined = handler.getClass().getAnnotation(Pipelined.class) != null;
+            state.setRequestState(pipelined ? MessageState.READY : MessageState.COMPLETED);
         }
     }
 
+    @Override
     public void responseReceived(
             final NHttpClientConnection conn) throws HttpException, IOException {
-        final State state = ensureNotNull(getState(conn));
+        final State state = getState(conn);
+        Asserts.notNull(state, "Connection state");
+        Asserts.check(state.getResponseState() == MessageState.READY,
+                "Unexpected request state %s", state.getResponseState());
+
         final HttpRequest request = state.getRequest();
         if (request == null) {
             throw new HttpException("Out of sequence response");
         }
-        final HttpAsyncClientExchangeHandler handler = ensureNotNull(getHandler(conn));
+        final HttpAsyncClientExchangeHandler handler = getHandler(conn);
+        Asserts.notNull(handler, "Client exchange handler");
         final HttpResponse response = conn.getHttpResponse();
 
         final int statusCode = response.getStatusLine().getStatusCode();
@@ -203,7 +261,7 @@ public class HttpAsyncRequestExecutor implements NHttpClientEventHandler {
                 final int timeout = state.getTimeout();
                 conn.setSocketTimeout(timeout);
                 conn.requestOutput();
-                state.setRequestState(MessageState.ACK);
+                state.setRequestState(MessageState.BODY_STREAM);
             }
             return;
         }
@@ -231,18 +289,24 @@ public class HttpAsyncRequestExecutor implements NHttpClientEventHandler {
         }
     }
 
+    @Override
     public void inputReady(
             final NHttpClientConnection conn,
             final ContentDecoder decoder) throws IOException, HttpException {
-        final State state = ensureNotNull(getState(conn));
-        final HttpAsyncClientExchangeHandler handler = ensureNotNull(getHandler(conn));
+        final State state = getState(conn);
+        Asserts.notNull(state, "Connection state");
+        Asserts.check(state.getResponseState() == MessageState.BODY_STREAM,
+                "Unexpected request state %s", state.getResponseState());
+
+        final HttpAsyncClientExchangeHandler handler = getHandler(conn);
+        Asserts.notNull(handler, "Client exchange handler");
         handler.consumeContent(decoder, conn);
-        state.setResponseState(MessageState.BODY_STREAM);
         if (decoder.isCompleted()) {
             processResponse(conn, state, handler);
         }
     }
 
+    @Override
     public void endOfInput(final NHttpClientConnection conn) throws IOException {
         final State state = getState(conn);
         if (state != null) {
@@ -268,6 +332,7 @@ public class HttpAsyncRequestExecutor implements NHttpClientEventHandler {
         conn.close();
     }
 
+    @Override
     public void timeout(
             final NHttpClientConnection conn) throws IOException {
         final State state = getState(conn);
@@ -277,6 +342,7 @@ public class HttpAsyncRequestExecutor implements NHttpClientEventHandler {
                 conn.setSocketTimeout(timeout);
                 conn.requestOutput();
                 state.setRequestState(MessageState.BODY_STREAM);
+                state.setTimeout(0);
                 return;
             } else {
                 state.invalidate();
@@ -307,26 +373,17 @@ public class HttpAsyncRequestExecutor implements NHttpClientEventHandler {
      * @param ex I/O exception thrown by {@link java.io.Closeable#close()}
      */
     protected void log(final Exception ex) {
+        this.exceptionLogger.log(ex);
     }
 
     private State getState(final NHttpConnection conn) {
         return (State) conn.getContext().getAttribute(HTTP_EXCHANGE_STATE);
     }
 
-    private State ensureNotNull(final State state) {
-        Asserts.notNull(state, "HTTP exchange state");
-        return state;
-    }
-
     private HttpAsyncClientExchangeHandler getHandler(final NHttpConnection conn) {
         return (HttpAsyncClientExchangeHandler) conn.getContext().getAttribute(HTTP_HANDLER);
     }
 
-    private HttpAsyncClientExchangeHandler ensureNotNull(final HttpAsyncClientExchangeHandler handler) {
-        Asserts.notNull(handler, "HTTP exchange handler");
-        return handler;
-    }
-
     private void shutdownConnection(final NHttpConnection conn) {
         try {
             conn.shutdown();
@@ -353,7 +410,14 @@ public class HttpAsyncRequestExecutor implements NHttpClientEventHandler {
             conn.close();
         }
         handler.responseCompleted();
-        state.reset();
+
+        final boolean pipelined = handler.getClass().getAnnotation(Pipelined.class) != null;
+        if (!pipelined) {
+            state.setRequestState(MessageState.READY);
+            state.setRequest(null);
+        }
+        state.setResponseState(MessageState.READY);
+        state.setResponse(null);
         if (!handler.isDone() && conn.isOpen()) {
             conn.requestOutput();
         }
@@ -434,14 +498,6 @@ public class HttpAsyncRequestExecutor implements NHttpClientEventHandler {
             this.timeout = timeout;
         }
 
-        public void reset() {
-            this.responseState = MessageState.READY;
-            this.requestState = MessageState.READY;
-            this.response = null;
-            this.request = null;
-            this.timeout = 0;
-        }
-
         public boolean isValid() {
             return this.valid;
         }
