@@ -26,11 +26,14 @@
  */
 package org.apache.http.nio.protocol;
 
+import java.io.Closeable;
 import java.io.IOException;
+import java.util.List;
 import java.util.concurrent.Future;
 
 import org.apache.http.ConnectionClosedException;
 import org.apache.http.ConnectionReuseStrategy;
+import org.apache.http.ExceptionLogger;
 import org.apache.http.HttpHost;
 import org.apache.http.annotation.Immutable;
 import org.apache.http.concurrent.BasicFuture;
@@ -46,7 +49,7 @@ import org.apache.http.protocol.HttpProcessor;
 import org.apache.http.util.Args;
 
 /**
- * <tt>HttpAsyncRequester</tt> is a utility class that can be used
+ * {@code HttpAsyncRequester} is a utility class that can be used
  * in conjunction with {@link HttpAsyncRequestExecutor} to initiate execution
  * of asynchronous HTTP requests.
  *
@@ -60,6 +63,7 @@ public class HttpAsyncRequester {
 
     private final HttpProcessor httpprocessor;
     private final ConnectionReuseStrategy connReuseStrategy;
+    private final ExceptionLogger exceptionLogger;
 
     /**
      * @deprecated (4.3) use {@link HttpAsyncRequester#HttpAsyncRequester(HttpProcessor,
@@ -73,6 +77,29 @@ public class HttpAsyncRequester {
         this(httpprocessor, reuseStrategy);
     }
 
+    /**
+     * Creates new instance of {@code HttpAsyncRequester}.
+     * @param httpprocessor HTTP protocol processor.
+     * @param connReuseStrategy Connection re-use strategy. If {@code null}
+     *   {@link DefaultConnectionReuseStrategy#INSTANCE} will be used.
+     * @param exceptionLogger Exception logger. If {@code null}
+     *   {@link ExceptionLogger#NO_OP} will be used. Please note that the exception
+     *   logger will be only used to log I/O exception thrown while closing
+     *   {@link java.io.Closeable} objects (such as {@link org.apache.http.HttpConnection}).
+     *
+     * @since 4.4
+     */
+    public HttpAsyncRequester(
+            final HttpProcessor httpprocessor,
+            final ConnectionReuseStrategy connReuseStrategy,
+            final ExceptionLogger exceptionLogger) {
+        super();
+        this.httpprocessor = Args.notNull(httpprocessor, "HTTP processor");
+        this.connReuseStrategy = connReuseStrategy != null ? connReuseStrategy :
+                DefaultConnectionReuseStrategy.INSTANCE;
+        this.exceptionLogger = exceptionLogger != null ? exceptionLogger : ExceptionLogger.NO_OP;
+    }
+
     /**
      * Creates new instance of HttpAsyncRequester.
      *
@@ -81,10 +108,7 @@ public class HttpAsyncRequester {
     public HttpAsyncRequester(
             final HttpProcessor httpprocessor,
             final ConnectionReuseStrategy connReuseStrategy) {
-        super();
-        this.httpprocessor = Args.notNull(httpprocessor, "HTTP processor");
-        this.connReuseStrategy = connReuseStrategy != null ? connReuseStrategy :
-            DefaultConnectionReuseStrategy.INSTANCE;
+        this(httpprocessor, connReuseStrategy, (ExceptionLogger) null);
     }
 
     /**
@@ -100,8 +124,8 @@ public class HttpAsyncRequester {
      * Initiates asynchronous HTTP request execution.
      *
      * @param <T> the result type of request execution.
-     * @param requestProducer request producer callback.
-     * @param responseConsumer response consumer callaback.
+     * @param requestProducer request producer.
+     * @param responseConsumer response consumer.
      * @param conn underlying HTTP connection.
      * @param context HTTP context
      * @param callback future callback.
@@ -143,8 +167,8 @@ public class HttpAsyncRequester {
      * Initiates asynchronous HTTP request execution.
      *
      * @param <T> the result type of request execution.
-     * @param requestProducer request producer callback.
-     * @param responseConsumer response consumer callaback.
+     * @param requestProducer request producer.
+     * @param responseConsumer response consumer.
      * @param conn underlying HTTP connection.
      * @param context HTTP context
      * @return future representing pending completion of the operation.
@@ -161,8 +185,8 @@ public class HttpAsyncRequester {
      * Initiates asynchronous HTTP request execution.
      *
      * @param <T> the result type of request execution.
-     * @param requestProducer request producer callback.
-     * @param responseConsumer response consumer callaback.
+     * @param requestProducer request producer.
+     * @param responseConsumer response consumer.
      * @param conn underlying HTTP connection.
      * @return future representing pending completion of the operation.
      */
@@ -178,8 +202,8 @@ public class HttpAsyncRequester {
      *
      * @param <T> the result type of request execution.
      * @param <E> the connection pool entry type.
-     * @param requestProducer request producer callback.
-     * @param responseConsumer response consumer callaback.
+     * @param requestProducer request producer.
+     * @param responseConsumer response consumer.
      * @param connPool pool of persistent reusable connections.
      * @param context HTTP context
      * @param callback future callback.
@@ -202,14 +226,47 @@ public class HttpAsyncRequester {
         return future;
     }
 
+    /**
+     * Initiates asynchronous pipelined HTTP request execution.
+     *
+     * @param <T> the result type of request execution.
+     * @param <E> the connection pool entry type.
+     * @param target target host.
+     * @param requestProducers list of request producers.
+     * @param responseConsumers list of response consumers.
+     * @param connPool pool of persistent reusable connections.
+     * @param context HTTP context
+     * @param callback future callback.
+     * @return future representing pending completion of the operation.
+     *
+     * @since 4.4
+     */
+    public <T, E extends PoolEntry<HttpHost, NHttpClientConnection>> Future<List<T>> executePipelined(
+            final HttpHost target,
+            final List<? extends HttpAsyncRequestProducer> requestProducers,
+            final List<? extends HttpAsyncResponseConsumer<T>> responseConsumers,
+            final ConnPool<HttpHost, E> connPool,
+            final HttpContext context,
+            final FutureCallback<List<T>> callback) {
+        Args.notNull(target, "HTTP target");
+        Args.notEmpty(requestProducers, "Request producer list");
+        Args.notEmpty(responseConsumers, "Response consumer list");
+        Args.notNull(connPool, "HTTP connection pool");
+        Args.notNull(context, "HTTP context");
+        final BasicFuture<List<T>> future = new BasicFuture<List<T>>(callback);
+        connPool.lease(target, null, new ConnPipelinedRequestCallback<T, E>(
+                future, requestProducers, responseConsumers, connPool, context));
+        return future;
+    }
+
     /**
      * Initiates asynchronous HTTP request execution. This method automatically releases
      * the given pool entry once request execution is completed (successfully or unsuccessfully).
      *
      * @param <T> the result type of request execution.
      * @param <E> the connection pool entry type.
-     * @param requestProducer request producer callback.
-     * @param responseConsumer response consumer callaback.
+     * @param requestProducer request producer.
+     * @param responseConsumer response consumer.
      * @param poolEntry leased pool entry. It will be automatically released
      *   back to the pool when execution is completed.
      * @param connPool pool of persistent reusable connections.
@@ -242,13 +299,53 @@ public class HttpAsyncRequester {
         return future;
     }
 
+    /**
+     * Initiates asynchronous pipelined HTTP request execution. This method automatically releases
+     * the given pool entry once request execution is completed (successfully or unsuccessfully).
+     *
+     * @param <T> the result type of request execution.
+     * @param <E> the connection pool entry type.
+     * @param requestProducers list of request producers.
+     * @param responseConsumers list of response consumers.
+     * @param poolEntry leased pool entry. It will be automatically released
+     *   back to the pool when execution is completed.
+     * @param connPool pool of persistent reusable connections.
+     * @param context HTTP context
+     * @param callback future callback.
+     * @return future representing pending completion of the operation.
+     *
+     * @since 4.4
+     */
+    public <T, E extends PoolEntry<HttpHost, NHttpClientConnection>> Future<List<T>> executePipelined(
+            final List<HttpAsyncRequestProducer> requestProducers,
+            final List<HttpAsyncResponseConsumer<T>> responseConsumers,
+            final E poolEntry,
+            final ConnPool<HttpHost, E> connPool,
+            final HttpContext context,
+            final FutureCallback<List<T>> callback) {
+        Args.notEmpty(requestProducers, "Request producer list");
+        Args.notEmpty(responseConsumers, "Response consumer list");
+        Args.notNull(connPool, "HTTP connection pool");
+        Args.notNull(poolEntry, "Pool entry");
+        Args.notNull(context, "HTTP context");
+        final BasicFuture<List<T>> future = new BasicFuture<List<T>>(callback);
+        final NHttpClientConnection conn = poolEntry.getConnection();
+        final PipeliningClientExchangeHandler<T> handler = new PipeliningClientExchangeHandler<T>(
+                requestProducers, responseConsumers,
+                new RequestExecutionCallback<List<T>, E>(future, poolEntry, connPool),
+                context, conn,
+                this.httpprocessor, this.connReuseStrategy);
+        initExection(handler, conn);
+        return future;
+    }
+
     /**
      * Initiates asynchronous HTTP request execution.
      *
      * @param <T> the result type of request execution.
      * @param <E> the connection pool entry type.
-     * @param requestProducer request producer callback.
-     * @param responseConsumer response consumer callaback.
+     * @param requestProducer request producer.
+     * @param responseConsumer response consumer.
      * @param connPool pool of persistent reusable connections.
      * @param context HTTP context
      * @return future representing pending completion of the operation.
@@ -266,8 +363,8 @@ public class HttpAsyncRequester {
      *
      * @param <T> the result type of request execution.
      * @param <E> the connection pool entry type.
-     * @param requestProducer request producer callback.
-     * @param responseConsumer response consumer callaback.
+     * @param requestProducer request producer.
+     * @param responseConsumer response consumer.
      * @param connPool pool of persistent reusable connections.
      * @return future representing pending completion of the operation.
      */
@@ -300,6 +397,7 @@ public class HttpAsyncRequester {
             this.context = context;
         }
 
+        @Override
         public void completed(final E result) {
             if (this.requestFuture.isDone()) {
                 this.connPool.release(result, true);
@@ -313,6 +411,7 @@ public class HttpAsyncRequester {
             initExection(handler, conn);
         }
 
+        @Override
         public void failed(final Exception ex) {
             try {
                 try {
@@ -325,6 +424,7 @@ public class HttpAsyncRequester {
             }
         }
 
+        @Override
         public void cancelled() {
             try {
                 try {
@@ -338,15 +438,84 @@ public class HttpAsyncRequester {
         }
 
         public void releaseResources() {
-            try {
-                this.requestProducer.close();
-            } catch (final IOException ioex) {
-                log(ioex);
+            close(requestProducer);
+            close(responseConsumer);
+        }
+
+    }
+
+    class ConnPipelinedRequestCallback<T, E extends PoolEntry<HttpHost, NHttpClientConnection>> implements FutureCallback<E> {
+
+        private final BasicFuture<List<T>> requestFuture;
+        private final List<? extends HttpAsyncRequestProducer> requestProducers;
+        private final List<? extends HttpAsyncResponseConsumer<T>> responseConsumers;
+        private final ConnPool<HttpHost, E> connPool;
+        private final HttpContext context;
+
+        ConnPipelinedRequestCallback(
+                final BasicFuture<List<T>> requestFuture,
+                final List<? extends HttpAsyncRequestProducer> requestProducers,
+                final List<? extends HttpAsyncResponseConsumer<T>> responseConsumers,
+                final ConnPool<HttpHost, E> connPool,
+                final HttpContext context) {
+            super();
+            this.requestFuture = requestFuture;
+            this.requestProducers = requestProducers;
+            this.responseConsumers = responseConsumers;
+            this.connPool = connPool;
+            this.context = context;
+        }
+
+        @Override
+        public void completed(final E result) {
+            if (this.requestFuture.isDone()) {
+                this.connPool.release(result, true);
+                return;
             }
+            final NHttpClientConnection conn = result.getConnection();
+            final PipeliningClientExchangeHandler<T> handler = new PipeliningClientExchangeHandler<T>(
+                    this.requestProducers, this.responseConsumers,
+                    new RequestExecutionCallback<List<T>, E>(this.requestFuture, result, this.connPool),
+                    this.context, conn, httpprocessor, connReuseStrategy);
+            initExection(handler, conn);
+        }
+
+        @Override
+        public void failed(final Exception ex) {
             try {
-                this.responseConsumer.close();
-            } catch (final IOException ioex) {
-                log(ioex);
+                try {
+                    for (HttpAsyncResponseConsumer<T> responseConsumer: this.responseConsumers) {
+                        responseConsumer.failed(ex);
+                    }
+                } finally {
+                    releaseResources();
+                }
+            } finally {
+                this.requestFuture.failed(ex);
+            }
+        }
+
+        @Override
+        public void cancelled() {
+            try {
+                try {
+                    for (HttpAsyncResponseConsumer<T> responseConsumer: this.responseConsumers) {
+                        responseConsumer.cancel();
+                    }
+                } finally {
+                    releaseResources();
+                }
+            } finally {
+                this.requestFuture.cancel(true);
+            }
+        }
+
+        public void releaseResources() {
+            for (HttpAsyncRequestProducer requestProducer: this.requestProducers) {
+                close(requestProducer);
+            }
+            for (HttpAsyncResponseConsumer<T> responseConsumer: this.responseConsumers) {
+                close(responseConsumer);
             }
         }
 
@@ -369,6 +538,7 @@ public class HttpAsyncRequester {
             this.connPool = connPool;
         }
 
+        @Override
         public void completed(final T result) {
             try {
                 this.connPool.release(this.poolEntry, true);
@@ -377,6 +547,7 @@ public class HttpAsyncRequester {
             }
         }
 
+        @Override
         public void failed(final Exception ex) {
             try {
                 this.connPool.release(this.poolEntry, false);
@@ -385,6 +556,7 @@ public class HttpAsyncRequester {
             }
         }
 
+        @Override
         public void cancelled() {
             try {
                 this.connPool.release(this.poolEntry, false);
@@ -403,6 +575,15 @@ public class HttpAsyncRequester {
      * @param ex I/O exception thrown by {@link java.io.Closeable#close()}
      */
     protected void log(final Exception ex) {
+        this.exceptionLogger.log(ex);
+    }
+
+    private void close(final Closeable closeable) {
+        try {
+            closeable.close();
+        } catch (IOException ex) {
+            log(ex);
+        }
     }
 
 }
