@@ -56,6 +56,7 @@ import org.apache.http.impl.io.ChunkedInputStream;
 import org.apache.http.impl.io.ChunkedOutputStream;
 import org.apache.http.impl.io.ContentLengthInputStream;
 import org.apache.http.impl.io.ContentLengthOutputStream;
+import org.apache.http.impl.io.EmptyInputStream;
 import org.apache.http.impl.io.HttpTransportMetricsImpl;
 import org.apache.http.impl.io.IdentityInputStream;
 import org.apache.http.impl.io.IdentityOutputStream;
@@ -79,6 +80,7 @@ public class BHttpConnectionBase implements HttpConnection, HttpInetConnection {
 
     private final SessionInputBufferImpl inbuffer;
     private final SessionOutputBufferImpl outbuffer;
+    private final MessageConstraints messageConstraints;
     private final HttpConnectionMetricsImpl connMetrics;
     private final ContentLengthStrategy incomingContentStrategy;
     private final ContentLengthStrategy outgoingContentStrategy;
@@ -90,14 +92,14 @@ public class BHttpConnectionBase implements HttpConnection, HttpInetConnection {
      * @param buffersize buffer size. Must be a positive number.
      * @param fragmentSizeHint fragment size hint.
      * @param chardecoder decoder to be used for decoding HTTP protocol elements.
-     *   If <code>null</code> simple type cast will be used for byte to char conversion.
+     *   If {@code null} simple type cast will be used for byte to char conversion.
      * @param charencoder encoder to be used for encoding HTTP protocol elements.
-     *   If <code>null</code> simple type cast will be used for char to byte conversion.
-     * @param constraints Message constraints. If <code>null</code>
+     *   If {@code null} simple type cast will be used for char to byte conversion.
+     * @param messageConstraints Message constraints. If {@code null}
      *   {@link MessageConstraints#DEFAULT} will be used.
-     * @param incomingContentStrategy incoming content length strategy. If <code>null</code>
+     * @param incomingContentStrategy incoming content length strategy. If {@code null}
      *   {@link LaxContentLengthStrategy#INSTANCE} will be used.
-     * @param outgoingContentStrategy outgoing content length strategy. If <code>null</code>
+     * @param outgoingContentStrategy outgoing content length strategy. If {@code null}
      *   {@link StrictContentLengthStrategy#INSTANCE} will be used.
      */
     protected BHttpConnectionBase(
@@ -105,7 +107,7 @@ public class BHttpConnectionBase implements HttpConnection, HttpInetConnection {
             final int fragmentSizeHint,
             final CharsetDecoder chardecoder,
             final CharsetEncoder charencoder,
-            final MessageConstraints constraints,
+            final MessageConstraints messageConstraints,
             final ContentLengthStrategy incomingContentStrategy,
             final ContentLengthStrategy outgoingContentStrategy) {
         super();
@@ -113,9 +115,10 @@ public class BHttpConnectionBase implements HttpConnection, HttpInetConnection {
         final HttpTransportMetricsImpl inTransportMetrics = new HttpTransportMetricsImpl();
         final HttpTransportMetricsImpl outTransportMetrics = new HttpTransportMetricsImpl();
         this.inbuffer = new SessionInputBufferImpl(inTransportMetrics, buffersize, -1,
-                constraints != null ? constraints : MessageConstraints.DEFAULT, chardecoder);
+                messageConstraints != null ? messageConstraints : MessageConstraints.DEFAULT, chardecoder);
         this.outbuffer = new SessionOutputBufferImpl(outTransportMetrics, buffersize, fragmentSizeHint,
                 charencoder);
+        this.messageConstraints = messageConstraints;
         this.connMetrics = new HttpConnectionMetricsImpl(inTransportMetrics, outTransportMetrics);
         this.incomingContentStrategy = incomingContentStrategy != null ? incomingContentStrategy :
             LaxContentLengthStrategy.INSTANCE;
@@ -146,9 +149,9 @@ public class BHttpConnectionBase implements HttpConnection, HttpInetConnection {
     /**
      * Binds this connection to the given {@link Socket}. This socket will be
      * used by the connection to send and receive data.
-     * <p/>
+     * <p>
      * After this method's execution the connection status will be reported
-     * as open and the {@link #isOpen()} will return <code>true</code>.
+     * as open and the {@link #isOpen()} will return {@code true}.
      *
      * @param socket the socket.
      * @throws IOException in case of an I/O error.
@@ -172,6 +175,7 @@ public class BHttpConnectionBase implements HttpConnection, HttpInetConnection {
         this.outbuffer.flush();
     }
 
+    @Override
     public boolean isOpen() {
         return this.socketHolder.get() != null;
     }
@@ -201,9 +205,11 @@ public class BHttpConnectionBase implements HttpConnection, HttpInetConnection {
             final long len,
             final SessionInputBuffer inbuffer) {
         if (len == ContentLengthStrategy.CHUNKED) {
-            return new ChunkedInputStream(inbuffer);
+            return new ChunkedInputStream(inbuffer, this.messageConstraints);
         } else if (len == ContentLengthStrategy.IDENTITY) {
             return new IdentityInputStream(inbuffer);
+        } else if (len == 0L) {
+            return EmptyInputStream.INSTANCE;
         } else {
             return new ContentLengthInputStream(inbuffer, len);
         }
@@ -239,26 +245,31 @@ public class BHttpConnectionBase implements HttpConnection, HttpInetConnection {
         return entity;
     }
 
+    @Override
     public InetAddress getLocalAddress() {
         final Socket socket = this.socketHolder.get();
         return socket != null ? socket.getLocalAddress() : null;
     }
 
+    @Override
     public int getLocalPort() {
         final Socket socket = this.socketHolder.get();
         return socket != null ? socket.getLocalPort() : -1;
     }
 
+    @Override
     public InetAddress getRemoteAddress() {
         final Socket socket = this.socketHolder.get();
         return socket != null ? socket.getInetAddress() : null;
     }
 
+    @Override
     public int getRemotePort() {
         final Socket socket = this.socketHolder.get();
         return socket != null ? socket.getPort() : -1;
     }
 
+    @Override
     public void setSocketTimeout(final int timeout) {
         final Socket socket = this.socketHolder.get();
         if (socket != null) {
@@ -272,6 +283,7 @@ public class BHttpConnectionBase implements HttpConnection, HttpInetConnection {
         }
     }
 
+    @Override
     public int getSocketTimeout() {
         final Socket socket = this.socketHolder.get();
         if (socket != null) {
@@ -285,13 +297,21 @@ public class BHttpConnectionBase implements HttpConnection, HttpInetConnection {
         }
     }
 
+    @Override
     public void shutdown() throws IOException {
         final Socket socket = this.socketHolder.getAndSet(null);
         if (socket != null) {
-            socket.close();
+            // force abortive close (RST)
+            try {
+                socket.setSoLinger(true, 0);
+            } catch (IOException ex) {
+            } finally {
+                socket.close();
+            }
         }
     }
 
+    @Override
     public void close() throws IOException {
         final Socket socket = this.socketHolder.getAndSet(null);
         if (socket != null) {
@@ -335,6 +355,7 @@ public class BHttpConnectionBase implements HttpConnection, HttpInetConnection {
         return this.inbuffer.hasBufferedData();
     }
 
+    @Override
     public boolean isStale() {
         if (!isOpen()) {
             return true;
@@ -357,6 +378,7 @@ public class BHttpConnectionBase implements HttpConnection, HttpInetConnection {
         this.connMetrics.incrementResponseCount();
     }
 
+    @Override
     public HttpConnectionMetrics getMetrics() {
         return this.connMetrics;
     }
